#pragma optimize("gsy",on)
#pragma comment(linker,"/RELEASE")
#pragma comment(linker,"/merge:.rdata=.data")
#pragma comment(linker,"/merge:.text=.data")
#pragma comment(linker,"/merge:.idata=.data")
#pragma comment(linker,"/merge:.bss=.data")
#pragma comment(linker,"/FILEALIGN:0x200")
/*#pragma comment(linker,"/merge:.idata$2=.data")
#pragma comment(linker,"/merge:.idata$3=.data")
#pragma comment(linker,"/merge:.idata$4=.data")
#pragma comment(linker,"/merge:.idata$6=.data")*/


/* 
x timing (delay,vars)
time end condition
move access
- drawing (loop,vars) (partially done) */


#define M_PI 3.14159265358979f	/* define M_PI if math.h doesn't */

#define _WIN32_WINNT 0x400	/* HACK: needed for newer DEVMODE */
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <math.h>


unsigned int (*xGetTickCount)() = (unsigned int (*)())0x77e7692e;


extern "C" __declspec(dllimport) BOOL __stdcall wglSwapBuffers(HDC);
extern "C" __declspec(dllimport) BOOL __stdcall wglSetPixelFormat(HDC,int,PIXELFORMATDESCRIPTOR*);




typedef unsigned char uint8;
typedef unsigned short uint16;
typedef short int16;


#define RANK(sq) ((sq) >> 3)
#define FILE(sq) ((sq) & 7)


enum
{
	pawn = 1, knight, bishop, rook, queen, king,
	bpawn = 0x81, bknight, bbishop, brook, bqueen, bking
};




static HDC hDC;




/* board */
static uint8 cboard[64] =
{
	rook,    knight,  bishop,  queen,   king,    bishop, knight,   rook,
	pawn,    pawn,    pawn,    pawn,    pawn,    pawn,   pawn,     pawn, 
	0,       0,       0,       0,       0,       0,      0,        0,
	0,       0,       0,       0,       0,       0,      0,        0,
	0,       0,       0,       0,       0,       0,      0,        0,
	0,       0,       0,       0,       0,       0,      0,        0,
	bpawn,   bpawn,   bpawn,   bpawn,   bpawn,   bpawn,   bpawn,   bpawn,
	brook,   bknight, bbishop, bqueen,  bking,   bbishop, bknight, brook
};


/************************************************************************************************
 * game logic
 ***********************************************************************************************/

/* move animation data */
static int f = 32, t = 32;		/* from/to square; pieces on these squares
								   are not drawn normally.
								   initial value: -> empty square b/c overwritten*/

static float fx,fz, ty;			/* position of moving/captured piece */



/*
 */
static void idle()
{
	static int move_start, move_len = 0;	/* start and length of move	animation [ms] */

	static float fdx, fdz, tdy;				/* move vector */

static int end;


	if(xGetTickCount() > end)
	{
		/*
		 * Morphy vs Isouard & Brunswick 1858
		 *
		 * format:
		 *   bit 15: game over (move is ignored)
		 *   bit 14: qside castle, f/t encodes king move (rook must be moved separately)
		 *   bits 6-11: 'from' square
		 *   bits 0-5: 'to' square
		 */
		static const uint16 mvs[] = { 796,3364,405,3307,731,3742,1764,1941,213,2788,346,4013,1361,3828,82,3242,166,3169,1185,2721,1697,3699,16642,3643,243,3827,451,3372,2163,2931,1145,3321,251,65535 };
		static const uint16* pmv = mvs;

		int mv = *pmv;
		if(mv >> 15)
			return;
		pmv++;

		cboard[t] = cboard[f];
		cboard[f] = 0; 

		t = mv & 0x3f;
		f = (mv >> 6) & 0x3f;
	
		if(mv & 0x4000)	/* qs castle */
			cboard[0] = 0, cboard[3] = rook;
	/*
		__asm
		{
			movzx	eax, byte ptr [f]
			and		al, 7
		}
*/

		fx = (float)FILE(f); fz = (float)RANK(f);
		ty = 0;
		fdx = (float)FILE(t)-fx; fdz = (float)RANK(t)-fz;


		tdy = -2.f / 400.f;	/* fall rate = 2 [units] in 400 [ms] */
		move_start = xGetTickCount();
		move_len = sqrtf(fdx*fdx+fdz*fdz)*200;	/* 200 ms per square width traveled */
		fdx /= move_len; fdz /= move_len;	/* allow pos = start + delta*elapsed */

		end = xGetTickCount() + move_len + (cboard[t]? 400 : 0) + 400;
	}


	int et = xGetTickCount()-move_start;

	et -= 400;
	if(et < 0)
		return;

	if(cboard[t])
	{
		int st = et - (move_len-200);
		if(st > 400)
			et -= 400;
		else if(st > 0)
		{
			ty = tdy*st;
			return;
		}
	}

	fx = FILE(f)+fdx*et;
	fz = RANK(f)+fdz*et;
}


/************************************************************************************************
 * graphics
 ***********************************************************************************************/

/* lighting / color parameters */
static const GLfloat light_dir[4] = { 10.f, 2.f, 4.f, 0.f };
static const GLfloat light_diff[4]   = { .90f, .90f, .90f, 1.f };	/* xxxxxxxxx: assimilate? */
static const GLfloat light_amb[4]    = { .95f, .95f, .95f, 1.f };
static const GLfloat mat_specular[4] = { .14f, .14f, .14f, 1.f };
static const GLubyte sq_color[2][4]    = { 89,89,89,255, 5,5,5,255 };
static const GLubyte piece_color[2][4] = { 89,89,89,255, 13,13,13,255 };


static void init_gl()
{
float m1[16],m2[16];

	/* projection / modelview matrices */
	glMatrixMode(GL_PROJECTION);
	gluPerspective(30., 1024./768., 10., 100.);
/*glGetFloatv(GL_PROJECTION_MATRIX,m1);

glLoadIdentity();
glFrustum(0,1024,0,768,1,100);*/
glGetFloatv(GL_PROJECTION_MATRIX,m2);

	glMatrixMode(GL_MODELVIEW);
	glTranslatef(-4, 2.99599, -20.0069);
	glRotatef(48.5035, 1,0,0);


	/* lighting */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_POSITION, light_dir);
///	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);

	/* piece material properties */
	glEnable(GL_COLOR_MATERIAL);	/* diffuse/ambient reflection set via glColor() */
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);

	glEnable(GL_DEPTH_TEST);
	/* NOTE: can't do backface culling b/c pieces don't have uniform winding */


	glEnable(GL_NORMALIZE);
}


/*
 * helper functions to create geometry from rotate/solid data,
 * based on Henk Kok's "chess" demo.
 */

static void solid(float* f, int cnt, float width)
{
	int i, j;
	float nx, ny, l;
	float sign = -1.f;

	glBegin(GL_QUAD_STRIP);
	for(i = 2; i < cnt-3; i += 2)
	{
//		if(f[i+3] == f[i+1] && f[i+2] == f[i])
//			continue;
//		nx = f[i-1]+f[i+3];
//		ny = f[i+2]-f[i];
//		l = 1.f/(float)sqrt(nx*nx+ny*ny);
//		glNormal3f(0, ny*l, nx*l);
		glVertex3f(width, f[i+1], f[i]);
		glVertex3f(-width, f[i+1], f[i]);
	}
	glEnd();


	glBegin(GL_TRIANGLE_STRIP);

solid1:
	glNormal3f(sign, 0, 0);
	i = 2; j = cnt-4;
	while(i < j)
	{
		while(f[i-1] == f[i+1] && f[i-2] == f[i] && i < j)
			i += 2;
		if(i < j)
			glVertex3f(sign*width, f[i+1], f[i]);
		i += 2;
		while(f[j-1] == f[j+1] && f[j-2] == f[j] && i < j)
			j -= 2;
		if(i < j)
			glVertex3f(sign*width, f[j+1], f[j]);
		j -= 2;
	}

	sign *= -1.f;
	if(sign > 0.f)
		goto solid1;

	glEnd();
}





	/*
	 * chess piece data (stored as indices into a table of fractions x / 2940.f).
	 * the i-th piece has size[i] entries for use with rotate();
	 * knight and king also have solid data after the rotate data.
	 */
	static const int16 tbl[] = {2100,0,150,2070,277,1985,362,1858,391,1707,1557,1430,1330,1259,210,364,720,384,469,658,315,588,104,1365,139,1085,455,490,979,770,595,735,350,664,244,3290,3219,175,3115,3044,2729,448,2449,2169,405,2134,1889,1854,1820,1749,517,1714,1680,1609,805,419,2240,559,839,511,4199,4094,3990,3919,3779,3499,3324,387,2975,2904,385,524,2555,525,644,2484,2415,2380,1119,279,4900,-139,4270,4549,-490,1960,609,2800,-210,3010,-350,2695,-595,-315,-559,1645,-455};
	static const uint8 idx[] =
	{
		10,1,0,2,3,4,5,6,7,8,9,6,10,4,11,4,11,8,12,8,13,14,13,14,13,15,16,15,16,17,16,17,16,18,18,19,20,19,14,21,14,21,22,19,22,19,1,19,1,1,1,1,
		23,24,23,24,23,24,25,24,25,26,25,26,25,27,28,27,29,30,30,31,32,31,32,31,14,33,34,33,22,31,22,31,1,31,1,1,1,1,
		35,1,36,37,38,22,39,22,39,32,40,41,42,20,43,20,43,44,45,20,0,20,0,20,46,20,46,8,47,15,48,15,48,27,49,50,51,27,52,14,53,14,53,32,54,32,54,17,54,17,54,30,30,31,32,31,32,31,14,33,34,33,22,31,22,31,1,31,1,1,1,1,
		40,1,0,55,0,55,0,55,56,55,56,57,56,57,56,57,56,55,46,55,46,17,46,17,46,27,58,27,58,59,58,59,58,30,30,31,32,31,32,31,14,33,34,33,22,31,22,31,1,31,1,1,1,1,
		60,1,60,14,61,24,62,14,63,14,63,17,64,17,64,30,64,57,64,57,65,57,65,26,66,67,39,67,39,26,68,17,69,17,69,17,40,70,40,71,72,71,72,73,72,74,75,32,76,32,77,26,78,27,78,71,78,54,30,58,57,58,32,58,32,29,79,58,14,58,14,58,1,58,1,1,1,1,
		63,1,63,14,63,17,64,30,64,67,39,67,39,26,68,17,69,17,69,17,40,70,40,71,72,71,72,73,72,74,75,32,76,32,76,32,77,26,78,27,78,71,78,54,30,58,57,58,32,58,32,29,79,58,14,58,14,58,1,58,1,1,1,1,
		31,78,32,78,32,78,57,85,86,0,17,42,1,87,88,89,88,89,90,91,90,91,92,42,92,42,92,56,92,56,81,0,81,0,93,49,94,95,94,95,96,78,1,78,
		11,64,24,64,24,64,24,80,24,80,81,80,81,80,81,64,81,64,1,64,
		21,82,27,82,27,82,27,83,27,83,84,83,84,83,84,82,84,82,1,82
	};

	static const uint8 rcnt[] = {0,24,17,34,25,37,30};

	static float data[442], *p = data;
	static const float _div = 1.f/2940.f;

float * xxx;
float* _f;

/*
 * draw piece centered on the square at (x,0,z), at a height of y
 * (the origin is at the top left corner of A8)
 */
static void draw_piece(int piece, float x, float y, float z)
{
	glColor4ubv(piece_color[piece >> 7]);

	glPushMatrix();
	z = 7.5f-z;
	glTranslatef(x+.5f, y, z);
	if(piece == bknight)
		glRotatef(180, 0, 1, 0);

int pc = piece&15;



_f = xxx;
int cnt=rcnt[pc];
	int i, j;
	float nx, ny;
	float a, c, s;

static float* qpos;
if(pc == queen)
qpos = _f;
/*
if(pc == king)
{
_f = qpos;
cnt = rcnt[queen];
}
/**/
	glBegin(GL_QUAD_STRIP);
	for(i = 0; i < cnt; i++)
	{
		a = 0.f;
		for(j = 0; j <= 8; j++)
		{
			__asm
			{
				fld			[a]
				fsincos
				fstp		[c]
				fstp		[s]

				mov			edx, [_f]
				fld			dword ptr [edx-4]
				fadd		dword ptr [edx+12]	; nx
				fld			dword ptr [edx+8]
				fsub		dword ptr [edx]		; ny
				fstp [ny]
				fstp [nx]
			}/**/
			c = cos(a); s = sin(a);
			nx = _f[1]+_f[3];		/* WAS f[-1]!!!!! */
			ny = _f[2]-_f[0];

			glNormal3f(c*nx, ny, s*nx);	/* gl normalizes */
			glVertex3f(c*_f[0], _f[1], s*_f[0]);
			glVertex3f(c*_f[2], _f[3], s*_f[2]);
			a += M_PI/4.f;
		}

		_f += 2;
	}
	glEnd();


	if(pc == knight)
		solid(data+358, 44, 0.08f);
	if(pc == king)
	{
		glRotatef(90, 0, 1, 0);
		solid(data+402, 20, 0.04f);
		solid(data+422, 20, 0.04f);
	}

	glPopMatrix();
}


static void display()
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

xxx = data+1;

	/* draw board */
	for(int pc = 0; pc < 8; pc++)
	{
		for(int sq = 0; sq < 64; sq++)
		{
			int x = sq & 7;
			int z = sq >> 3;

			if((cboard[sq]&0x7f) == pc)
			{

				/* board */
				glBegin(GL_QUADS);
				glNormal3b(0, 127, 0);
				glColor4ubv(sq_color[(x+z)&1]);
				glVertex3i(x,   0, z);
				glVertex3i(x+1, 0, z);
				glVertex3i(x+1, 0, z+1);
				glVertex3i(x,   0, z+1);
				glEnd();

				/* pieces */
				float _x = (float)FILE(sq), _y = 0, _z = (float)RANK(sq);
				if(sq == t)
					_y = ty;
				if(sq == f)
					_x = fx, _z = fz;
				draw_piece(cboard[sq], _x, _y, _z);
			}
		}
if(pc)
		xxx = _f+4;
	}

	SwapBuffers(hDC);
}


int main()
{
	HWND hWnd = CreateWindowEx(WS_EX_TOPMOST*0, "EDIT", NULL, WS_POPUP|WS_VISIBLE, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), 0, 0, 0, 0);
	hDC = GetDC(hWnd);

	SetPixelFormat(hDC,4,(PIXELFORMATDESCRIPTOR*)&hDC);
	wglMakeCurrent(hDC, wglCreateContext(hDC));

ShowCursor(0);

	init_gl();

	/* expand tbl/idx into floats */
	__asm
	{
		mov		ecx, 442
$1:
		movzx	eax, byte ptr [idx+ecx-1]
		fld		dword ptr [_div]
		fimul	word ptr [tbl+eax*2]
		fstp	dword ptr [data+ecx*4-4]
		loop	$1
	}
MSG msg;

	while(!GetAsyncKeyState(VK_ESCAPE))
	{
GetMessage(&msg,0,0,0);
		idle();
		display();
	}

	return 0;
}